diff --git a/bin/doctrine.bat b/bin/doctrine.bat
new file mode 100644
index 000000000..a9e8ceefd
--- /dev/null
+++ b/bin/doctrine.bat
@@ -0,0 +1,9 @@
+@echo off
+
+if "%PHPBIN%" == "" set PHPBIN=@php_bin@
+if not exist "%PHPBIN%" if "%PHP_PEAR_PHP_BIN%" neq "" goto USE_PEAR_PATH
+GOTO RUN
+:USE_PEAR_PATH
+set PHPBIN=%PHP_PEAR_PHP_BIN%
+:RUN
+"%PHPBIN%" "@bin_dir@\doctrine" %*
diff --git a/bin/doctrine.php b/bin/doctrine.php
index a94cdff67..701038488 100644
--- a/bin/doctrine.php
+++ b/bin/doctrine.php
@@ -1,4 +1,21 @@
.
+ */
require_once 'Doctrine/Common/ClassLoader.php';
@@ -19,7 +36,7 @@ if (file_exists($configFile)) {
}
require $configFile;
-
+
foreach ($GLOBALS as $helperSetCandidate) {
if ($helperSetCandidate instanceof \Symfony\Component\Console\Helper\HelperSet) {
$helperSet = $helperSetCandidate;
@@ -30,29 +47,4 @@ if (file_exists($configFile)) {
$helperSet = ($helperSet) ?: new \Symfony\Component\Console\Helper\HelperSet();
-$cli = new \Symfony\Component\Console\Application('Doctrine Command Line Interface', Doctrine\ORM\Version::VERSION);
-$cli->setCatchExceptions(true);
-$cli->setHelperSet($helperSet);
-$cli->addCommands(array(
- // DBAL Commands
- new \Doctrine\DBAL\Tools\Console\Command\RunSqlCommand(),
- new \Doctrine\DBAL\Tools\Console\Command\ImportCommand(),
-
- // ORM Commands
- new \Doctrine\ORM\Tools\Console\Command\ClearCache\MetadataCommand(),
- new \Doctrine\ORM\Tools\Console\Command\ClearCache\ResultCommand(),
- new \Doctrine\ORM\Tools\Console\Command\ClearCache\QueryCommand(),
- new \Doctrine\ORM\Tools\Console\Command\SchemaTool\CreateCommand(),
- new \Doctrine\ORM\Tools\Console\Command\SchemaTool\UpdateCommand(),
- new \Doctrine\ORM\Tools\Console\Command\SchemaTool\DropCommand(),
- new \Doctrine\ORM\Tools\Console\Command\EnsureProductionSettingsCommand(),
- new \Doctrine\ORM\Tools\Console\Command\ConvertDoctrine1SchemaCommand(),
- new \Doctrine\ORM\Tools\Console\Command\GenerateRepositoriesCommand(),
- new \Doctrine\ORM\Tools\Console\Command\GenerateEntitiesCommand(),
- new \Doctrine\ORM\Tools\Console\Command\GenerateProxiesCommand(),
- new \Doctrine\ORM\Tools\Console\Command\ConvertMappingCommand(),
- new \Doctrine\ORM\Tools\Console\Command\RunDqlCommand(),
- new \Doctrine\ORM\Tools\Console\Command\ValidateSchemaCommand(),
-
-));
-$cli->run();
+\Doctrine\ORM\Tools\Console\ConsoleRunner::run($helperSet);
diff --git a/build.properties.dev b/build.properties.dev
index e1ba8fb49..7e72d6f1e 100644
--- a/build.properties.dev
+++ b/build.properties.dev
@@ -1,10 +1,13 @@
version=2.0.0BETA2
+dependencies.common=2.0.0BETA4
+dependencies.dbal=2.0.0BETA4
stability=beta
build.dir=build
dist.dir=dist
report.dir=reports
log.archive.dir=logs
-svn.path=/usr/bin/svn
+project.pirum_dir=
+project.download_dir=
test.phpunit_configuration_file=
test.phpunit_generate_coverage=0
test.pmd_reports=0
diff --git a/build.xml b/build.xml
index 024b63c89..e4b7805e1 100644
--- a/build.xml
+++ b/build.xml
@@ -5,8 +5,6 @@
-->
-
-
@@ -24,6 +22,7 @@
+
-
+
-
-
+
+
+
-
+
-
+
-
+
-
-
+
+
@@ -141,8 +141,8 @@
-
-
+
+
DoctrineORM
Doctrine Object Relational Mapper
pear.doctrine-project.org
@@ -150,6 +150,7 @@
+
LGPL
@@ -157,17 +158,55 @@
+
+
script
-
-
-
+ Doctrine/Common/
+ Doctrine/DBAL/
+
+
+
-
-
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
\ No newline at end of file
diff --git a/doctrine-mapping.xsd b/doctrine-mapping.xsd
index 3a09f24b3..2dde4d9b8 100644
--- a/doctrine-mapping.xsd
+++ b/doctrine-mapping.xsd
@@ -63,7 +63,7 @@
-
+
@@ -73,7 +73,7 @@
-
+
@@ -252,7 +252,7 @@
-
+
@@ -264,7 +264,7 @@
-
+
@@ -279,7 +279,7 @@
-
+
@@ -295,11 +295,11 @@
-
+
-
\ No newline at end of file
+
diff --git a/lib/Doctrine/ORM/AbstractQuery.php b/lib/Doctrine/ORM/AbstractQuery.php
index 21974b792..dd9d1c81a 100644
--- a/lib/Doctrine/ORM/AbstractQuery.php
+++ b/lib/Doctrine/ORM/AbstractQuery.php
@@ -482,15 +482,6 @@ abstract class AbstractQuery
*/
public function execute($params = array(), $hydrationMode = null)
{
- // If there are still pending insertions in the UnitOfWork we need to flush
- // in order to guarantee a correct result.
- //TODO: Think this over. Its tricky. Not doing this can lead to strange results
- // potentially, but doing it could result in endless loops when querying during
- // a flush, i.e. inside an event listener.
- if ($this->_em->getUnitOfWork()->hasPendingInsertions()) {
- $this->_em->flush();
- }
-
if ($hydrationMode !== null) {
$this->setHydrationMode($hydrationMode);
}
@@ -562,9 +553,22 @@ abstract class AbstractQuery
if ($this->_resultCacheId) {
return $this->_resultCacheId;
} else {
+ $params = $this->_params;
+ foreach ($params AS $key => $value) {
+ if (is_object($value) && $this->_em->getMetadataFactory()->hasMetadataFor(get_class($value))) {
+ if ($this->_em->getUnitOfWork()->getEntityState($value) == UnitOfWork::STATE_MANAGED) {
+ $idValues = $this->_em->getUnitOfWork()->getEntityIdentifier($value);
+ } else {
+ $class = $this->_em->getClassMetadata(get_class($value));
+ $idValues = $class->getIdentifierValues($value);
+ }
+ $params[$key] = $idValues;
+ }
+ }
+
$sql = $this->getSql();
ksort($this->_hints);
- return md5(implode(";", (array)$sql) . var_export($this->_params, true) .
+ return md5(implode(";", (array)$sql) . var_export($params, true) .
var_export($this->_hints, true)."&hydrationMode=".$this->_hydrationMode);
}
}
@@ -583,6 +587,8 @@ abstract class AbstractQuery
*/
public function __clone()
{
- $this->free();
+ $this->_params = array();
+ $this->_paramTypes = array();
+ $this->_hints = array();
}
}
diff --git a/lib/Doctrine/ORM/Configuration.php b/lib/Doctrine/ORM/Configuration.php
index 5e5341d1d..0bc206d27 100644
--- a/lib/Doctrine/ORM/Configuration.php
+++ b/lib/Doctrine/ORM/Configuration.php
@@ -462,4 +462,25 @@ class Configuration extends \Doctrine\DBAL\Configuration
{
$this->_attributes['customHydrationModes'][$modeName] = $hydrator;
}
+
+ /**
+ * Set a class metadata factory.
+ *
+ * @param string $cmf
+ */
+ public function setClassMetadataFactoryName($cmfName)
+ {
+ $this->_attributes['classMetadataFactoryName'] = $cmfName;
+ }
+
+ /**
+ * @return string
+ */
+ public function getClassMetadataFactoryName()
+ {
+ if (!isset($this->_attributes['classMetadataFactoryName'])) {
+ $this->_attributes['classMetadataFactoryName'] = 'Doctrine\ORM\Mapping\ClassMetadataFactory';
+ }
+ return $this->_attributes['classMetadataFactoryName'];
+ }
}
\ No newline at end of file
diff --git a/lib/Doctrine/ORM/EntityManager.php b/lib/Doctrine/ORM/EntityManager.php
index 0cd887fe1..3af9cf3c3 100644
--- a/lib/Doctrine/ORM/EntityManager.php
+++ b/lib/Doctrine/ORM/EntityManager.php
@@ -118,8 +118,12 @@ class EntityManager
$this->conn = $conn;
$this->config = $config;
$this->eventManager = $eventManager;
- $this->metadataFactory = new ClassMetadataFactory($this);
+
+ $metadataFactoryClassName = $config->getClassMetadataFactoryName();
+ $this->metadataFactory = new $metadataFactoryClassName;
+ $this->metadataFactory->setEntityManager($this);
$this->metadataFactory->setCacheDriver($this->config->getMetadataCacheImpl());
+
$this->unitOfWork = new UnitOfWork($this);
$this->proxyFactory = new ProxyFactory($this,
$config->getProxyDir(),
@@ -352,11 +356,15 @@ class EntityManager
if ($entity = $this->unitOfWork->tryGetById($identifier, $class->rootEntityName)) {
return $entity;
}
- if ( ! is_array($identifier)) {
- $identifier = array($class->identifier[0] => $identifier);
+ if ($class->subClasses) {
+ $entity = $this->find($entityName, $identifier);
+ } else {
+ if ( ! is_array($identifier)) {
+ $identifier = array($class->identifier[0] => $identifier);
+ }
+ $entity = $this->proxyFactory->getProxy($class->name, $identifier);
+ $this->unitOfWork->registerManaged($entity, $identifier, array());
}
- $entity = $this->proxyFactory->getProxy($class->name, $identifier);
- $this->unitOfWork->registerManaged($entity, $identifier, array());
return $entity;
}
@@ -611,6 +619,16 @@ class EntityManager
}
}
+ /**
+ * Check if the Entity manager is open or closed.
+ *
+ * @return bool
+ */
+ public function isOpen()
+ {
+ return (!$this->closed);
+ }
+
/**
* Gets the UnitOfWork used by the EntityManager to coordinate operations.
*
diff --git a/lib/Doctrine/ORM/EntityRepository.php b/lib/Doctrine/ORM/EntityRepository.php
index 2e4a2191c..81680647c 100644
--- a/lib/Doctrine/ORM/EntityRepository.php
+++ b/lib/Doctrine/ORM/EntityRepository.php
@@ -133,7 +133,6 @@ class EntityRepository
/**
* Finds all entities in the repository.
*
- * @param int $hydrationMode
* @return array The entities.
*/
public function findAll()
@@ -144,8 +143,7 @@ class EntityRepository
/**
* Finds entities by a set of criteria.
*
- * @param string $column
- * @param string $value
+ * @param array $criteria
* @return array
*/
public function findBy(array $criteria)
@@ -156,8 +154,7 @@ class EntityRepository
/**
* Finds a single entity by a set of criteria.
*
- * @param string $column
- * @param string $value
+ * @param array $criteria
* @return object
*/
public function findOneBy(array $criteria)
@@ -188,13 +185,14 @@ class EntityRepository
);
}
- if ( ! isset($arguments[0])) {
+ if ( !isset($arguments[0])) {
+ // we dont even want to allow null at this point, because we cannot (yet) transform it into IS NULL.
throw ORMException::findByRequiresParameter($method.$by);
}
$fieldName = lcfirst(\Doctrine\Common\Util\Inflector::classify($by));
- if ($this->_class->hasField($fieldName)) {
+ if ($this->_class->hasField($fieldName) || $this->_class->hasAssociation($fieldName)) {
return $this->$method(array($fieldName => $arguments[0]));
} else {
throw ORMException::invalidFindByCall($this->_entityName, $fieldName, $method.$by);
diff --git a/lib/Doctrine/ORM/Event/LoadClassMetadataEventArgs.php b/lib/Doctrine/ORM/Event/LoadClassMetadataEventArgs.php
index 862152a99..f00520a20 100644
--- a/lib/Doctrine/ORM/Event/LoadClassMetadataEventArgs.php
+++ b/lib/Doctrine/ORM/Event/LoadClassMetadataEventArgs.php
@@ -4,6 +4,9 @@ namespace Doctrine\ORM\Event;
use Doctrine\Common\EventArgs;
+use Doctrine\ORM\Mapping\ClassMetadataInfo;
+use Doctrine\ORM\EntityManager;
+
/**
* Class that holds event arguments for a loadMetadata event.
*
@@ -12,16 +15,40 @@ use Doctrine\Common\EventArgs;
*/
class LoadClassMetadataEventArgs extends EventArgs
{
- private $_classMetadata;
+ /**
+ * @var ClassMetadata
+ */
+ private $classMetadata;
- public function __construct(\Doctrine\ORM\Mapping\ClassMetadata $classMetadata)
+ /**
+ * @var EntityManager
+ */
+ private $em;
+
+ /**
+ * @param ClassMetadataInfo $classMetadata
+ * @param EntityManager $em
+ */
+ public function __construct(ClassMetadataInfo $classMetadata, EntityManager $em)
{
- $this->_classMetadata = $classMetadata;
+ $this->classMetadata = $classMetadata;
+ $this->em = $em;
}
+ /**
+ * @return ClassMetadataInfo
+ */
public function getClassMetadata()
{
- return $this->_classMetadata;
+ return $this->classMetadata;
+ }
+
+ /**
+ * @return EntityManager
+ */
+ public function getEntityManager()
+ {
+ return $this->em;
}
}
diff --git a/lib/Doctrine/ORM/Internal/Hydration/ObjectHydrator.php b/lib/Doctrine/ORM/Internal/Hydration/ObjectHydrator.php
index a13119aea..202fdc7ff 100644
--- a/lib/Doctrine/ORM/Internal/Hydration/ObjectHydrator.php
+++ b/lib/Doctrine/ORM/Internal/Hydration/ObjectHydrator.php
@@ -145,12 +145,12 @@ class ObjectHydrator extends AbstractHydrator
{
$oid = spl_object_hash($entity);
$relation = $class->associationMappings[$fieldName];
-
+
$value = $class->reflFields[$fieldName]->getValue($entity);
if ($value === null) {
$value = new ArrayCollection;
}
-
+
if ( ! $value instanceof PersistentCollection) {
$value = new PersistentCollection(
$this->_em,
@@ -161,17 +161,19 @@ class ObjectHydrator extends AbstractHydrator
$class->reflFields[$fieldName]->setValue($entity, $value);
$this->_uow->setOriginalEntityProperty($oid, $fieldName, $value);
$this->_initializedCollections[$oid . $fieldName] = $value;
- } else if (isset($this->_hints[Query::HINT_REFRESH])) {
- // Is already PersistentCollection, but REFRESH
+ } else if (isset($this->_hints[Query::HINT_REFRESH]) ||
+ isset($this->_hints['fetched'][$class->name][$fieldName]) &&
+ ! $value->isInitialized()) {
+ // Is already PersistentCollection, but either REFRESH or FETCH-JOIN and UNINITIALIZED!
$value->setDirty(false);
$value->setInitialized(true);
$value->unwrap()->clear();
$this->_initializedCollections[$oid . $fieldName] = $value;
} else {
- // Is already PersistentCollection, and DONT REFRESH
+ // Is already PersistentCollection, and DON'T REFRESH or FETCH-JOIN!
$this->_existingCollections[$oid . $fieldName] = $value;
}
-
+
return $value;
}
@@ -269,6 +271,9 @@ class ObjectHydrator extends AbstractHydrator
// It's a joined result
$parentAlias = $this->_rsm->parentAliasMap[$dqlAlias];
+ // we need the $path to save into the identifier map which entities were already
+ // seen for this parent-child relationship
+ $path = $parentAlias . '.' . $dqlAlias;
// Get a reference to the parent object to which the joined element belongs.
if ($this->_rsm->isMixed && isset($this->_rootAliases[$parentAlias])) {
@@ -298,8 +303,8 @@ class ObjectHydrator extends AbstractHydrator
$reflFieldValue = $this->_initRelatedCollection($parentObject, $parentClass, $relationField);
}
- $indexExists = isset($this->_identifierMap[$dqlAlias][$id[$dqlAlias]]);
- $index = $indexExists ? $this->_identifierMap[$dqlAlias][$id[$dqlAlias]] : false;
+ $indexExists = isset($this->_identifierMap[$path][$id[$parentAlias]][$id[$dqlAlias]]);
+ $index = $indexExists ? $this->_identifierMap[$path][$id[$parentAlias]][$id[$dqlAlias]] : false;
$indexIsValid = $index !== false ? isset($reflFieldValue[$index]) : false;
if ( ! $indexExists || ! $indexIsValid) {
@@ -317,11 +322,11 @@ class ObjectHydrator extends AbstractHydrator
$field = $this->_rsm->indexByMap[$dqlAlias];
$indexValue = $this->_ce[$entityName]->reflFields[$field]->getValue($element);
$reflFieldValue->hydrateSet($indexValue, $element);
- $this->_identifierMap[$dqlAlias][$id[$dqlAlias]] = $indexValue;
+ $this->_identifierMap[$path][$id[$parentAlias]][$id[$dqlAlias]] = $indexValue;
} else {
$reflFieldValue->hydrateAdd($element);
$reflFieldValue->last();
- $this->_identifierMap[$dqlAlias][$id[$dqlAlias]] = $reflFieldValue->key();
+ $this->_identifierMap[$path][$id[$parentAlias]][$id[$dqlAlias]] = $reflFieldValue->key();
}
// Update result pointer
$this->_resultPointers[$dqlAlias] = $element;
@@ -332,6 +337,7 @@ class ObjectHydrator extends AbstractHydrator
}
} else if ( ! $reflField->getValue($parentObject)) {
$coll = new PersistentCollection($this->_em, $this->_ce[$entityName], new ArrayCollection);
+ $coll->setOwner($parentObject, $relation);
$reflField->setValue($parentObject, $coll);
$this->_uow->setOriginalEntityProperty($oid, $relationField, $coll);
}
diff --git a/lib/Doctrine/ORM/Mapping/ClassMetadataFactory.php b/lib/Doctrine/ORM/Mapping/ClassMetadataFactory.php
index 400970d6d..0f0fb17aa 100644
--- a/lib/Doctrine/ORM/Mapping/ClassMetadataFactory.php
+++ b/lib/Doctrine/ORM/Mapping/ClassMetadataFactory.php
@@ -74,11 +74,9 @@ class ClassMetadataFactory
private $initialized = false;
/**
- * Creates a new factory instance that uses the given metadata driver implementation.
- *
- * @param $driver The metadata driver to use.
+ * @param EntityManager $$em
*/
- public function __construct(EntityManager $em)
+ public function setEntityManager(EntityManager $em)
{
$this->em = $em;
}
@@ -262,15 +260,19 @@ class ClassMetadataFactory
$class = $this->newClassMetadataInstance($className);
if ($parent) {
- $class->setInheritanceType($parent->inheritanceType);
- $class->setDiscriminatorColumn($parent->discriminatorColumn);
+ if (!$parent->isMappedSuperclass) {
+ $class->setInheritanceType($parent->inheritanceType);
+ $class->setDiscriminatorColumn($parent->discriminatorColumn);
+ }
$class->setIdGeneratorType($parent->generatorType);
$this->addInheritedFields($class, $parent);
$this->addInheritedRelations($class, $parent);
$class->setIdentifier($parent->identifier);
$class->setVersioned($parent->isVersioned);
$class->setVersionField($parent->versionField);
- $class->setDiscriminatorMap($parent->discriminatorMap);
+ if (!$parent->isMappedSuperclass) {
+ $class->setDiscriminatorMap($parent->discriminatorMap);
+ }
$class->setLifecycleCallbacks($parent->lifecycleCallbacks);
$class->setChangeTrackingPolicy($parent->changeTrackingPolicy);
}
@@ -302,6 +304,17 @@ class ClassMetadataFactory
$this->completeIdGeneratorMapping($class);
}
+ if ($parent && $parent->isInheritanceTypeSingleTable()) {
+ $class->setPrimaryTable($parent->table);
+ }
+
+ $class->setParentClasses($visited);
+
+ if ($this->evm->hasListeners(Events::loadClassMetadata)) {
+ $eventArgs = new \Doctrine\ORM\Event\LoadClassMetadataEventArgs($class, $this->em);
+ $this->evm->dispatchEvent(Events::loadClassMetadata, $eventArgs);
+ }
+
// verify inheritance
if (!$parent && !$class->isMappedSuperclass && !$class->isInheritanceTypeNone()) {
if (count($class->discriminatorMap) == 0) {
@@ -312,17 +325,6 @@ class ClassMetadataFactory
}
}
- if ($parent && $parent->isInheritanceTypeSingleTable()) {
- $class->setPrimaryTable($parent->table);
- }
-
- $class->setParentClasses($visited);
-
- if ($this->evm->hasListeners(Events::loadClassMetadata)) {
- $eventArgs = new \Doctrine\ORM\Event\LoadClassMetadataEventArgs($class);
- $this->evm->dispatchEvent(Events::loadClassMetadata, $eventArgs);
- }
-
$this->loadedMetadata[$className] = $class;
$parent = $class;
@@ -379,6 +381,13 @@ class ClassMetadataFactory
private function addInheritedRelations(ClassMetadata $subClass, ClassMetadata $parentClass)
{
foreach ($parentClass->associationMappings as $field => $mapping) {
+ if ($parentClass->isMappedSuperclass) {
+ if ($mapping['type'] & ClassMetadata::TO_MANY) {
+ throw MappingException::illegalToManyAssocationOnMappedSuperclass($parentClass->name, $field);
+ }
+ $mapping['sourceEntity'] = $subClass->name;
+ }
+
//$subclassMapping = $mapping;
if ( ! isset($mapping['inherited']) && ! $parentClass->isMappedSuperclass) {
$mapping['inherited'] = $parentClass->name;
diff --git a/lib/Doctrine/ORM/Mapping/ClassMetadataInfo.php b/lib/Doctrine/ORM/Mapping/ClassMetadataInfo.php
index 971e2c980..53042d015 100644
--- a/lib/Doctrine/ORM/Mapping/ClassMetadataInfo.php
+++ b/lib/Doctrine/ORM/Mapping/ClassMetadataInfo.php
@@ -598,9 +598,10 @@ class ClassMetadataInfo
/**
* Gets the mapping of an association.
*
+ * @see ClassMetadataInfo::$associationMappings
* @param string $fieldName The field name that represents the association in
* the object model.
- * @return Doctrine\ORM\Mapping\AssociationMapping The mapping.
+ * @return array The mapping.
*/
public function getAssociationMapping($fieldName)
{
@@ -669,6 +670,10 @@ class ClassMetadataInfo
// Complete id mapping
if (isset($mapping['id']) && $mapping['id'] === true) {
+ if ($this->versionField == $mapping['fieldName']) {
+ throw MappingException::cannotVersionIdField($this->name, $mapping['fieldName']);
+ }
+
if ( ! in_array($mapping['fieldName'], $this->identifier)) {
$this->identifier[] = $mapping['fieldName'];
}
@@ -947,6 +952,7 @@ class ClassMetadataInfo
public function setIdentifier(array $identifier)
{
$this->identifier = $identifier;
+ $this->isIdentifierComposite = (count($this->identifier) > 1);
}
/**
@@ -1485,6 +1491,13 @@ class ClassMetadataInfo
if ( ! isset($columnDef['fieldName'])) {
$columnDef['fieldName'] = $columnDef['name'];
}
+ if ( ! isset($columnDef['type'])) {
+ $columnDef['type'] = "string";
+ }
+ if (in_array($columnDef['type'], array("boolean", "array", "object", "datetime", "time", "date"))) {
+ throw MappingException::invalidDiscriminatorColumnType($this->name, $columnDef['type']);
+ }
+
$this->discriminatorColumn = $columnDef;
}
}
diff --git a/lib/Doctrine/ORM/Mapping/Driver/AbstractFileDriver.php b/lib/Doctrine/ORM/Mapping/Driver/AbstractFileDriver.php
index 637971cb5..457b7cda7 100644
--- a/lib/Doctrine/ORM/Mapping/Driver/AbstractFileDriver.php
+++ b/lib/Doctrine/ORM/Mapping/Driver/AbstractFileDriver.php
@@ -155,7 +155,7 @@ abstract class AbstractFileDriver implements Driver
if ($this->_paths) {
foreach ((array) $this->_paths as $path) {
if ( ! is_dir($path)) {
- throw MappingException::fileMappingDriversRequireConfiguredDirectoryPath();
+ throw MappingException::fileMappingDriversRequireConfiguredDirectoryPath($path);
}
$iterator = new \RecursiveIteratorIterator(
diff --git a/lib/Doctrine/ORM/Mapping/Driver/AnnotationDriver.php b/lib/Doctrine/ORM/Mapping/Driver/AnnotationDriver.php
index be24e69db..81dcc90da 100644
--- a/lib/Doctrine/ORM/Mapping/Driver/AnnotationDriver.php
+++ b/lib/Doctrine/ORM/Mapping/Driver/AnnotationDriver.php
@@ -179,6 +179,8 @@ class AnnotationDriver implements Driver
'type' => $discrColumnAnnot->type,
'length' => $discrColumnAnnot->length
));
+ } else {
+ $metadata->setDiscriminatorColumn(array('name' => 'dtype', 'type' => 'string', 'length' => 255));
}
// Evaluate DiscriminatorMap annotation
@@ -443,7 +445,7 @@ class AnnotationDriver implements Driver
foreach ($this->_paths as $path) {
if ( ! is_dir($path)) {
- throw MappingException::fileMappingDriversRequireConfiguredDirectoryPath();
+ throw MappingException::fileMappingDriversRequireConfiguredDirectoryPath($path);
}
$iterator = new \RecursiveIteratorIterator(
diff --git a/lib/Doctrine/ORM/Mapping/Driver/DatabaseDriver.php b/lib/Doctrine/ORM/Mapping/Driver/DatabaseDriver.php
index 5dbe6e431..c6c4547b6 100644
--- a/lib/Doctrine/ORM/Mapping/Driver/DatabaseDriver.php
+++ b/lib/Doctrine/ORM/Mapping/Driver/DatabaseDriver.php
@@ -48,7 +48,7 @@ class DatabaseDriver implements Driver
*/
private $tables = null;
- private $classes = array();
+ private $classToTableNames = array();
/**
* @var array
@@ -73,11 +73,11 @@ class DatabaseDriver implements Driver
}
foreach ($this->_sm->listTableNames() as $tableName) {
- $tables[strtolower($tableName)] = $this->_sm->listTableDetails($tableName);
+ $tables[$tableName] = $this->_sm->listTableDetails($tableName);
}
$this->tables = array();
- foreach ($tables AS $name => $table) {
+ foreach ($tables AS $tableName => $table) {
/* @var $table Table */
if ($this->_sm->getDatabasePlatform()->supportsForeignKeyConstraints()) {
$foreignKeys = $table->getForeignKeys();
@@ -89,21 +89,23 @@ class DatabaseDriver implements Driver
foreach ($foreignKeys AS $foreignKey) {
$allForeignKeyColumns = array_merge($allForeignKeyColumns, $foreignKey->getLocalColumns());
}
-
+
$pkColumns = $table->getPrimaryKey()->getColumns();
sort($pkColumns);
sort($allForeignKeyColumns);
if ($pkColumns == $allForeignKeyColumns) {
if (count($table->getForeignKeys()) > 2) {
- throw new \InvalidArgumentException("ManyToMany table '" . $name . "' with more or less than two foreign keys are not supported by the Database Reverese Engineering Driver.");
+ throw new \InvalidArgumentException("ManyToMany table '" . $tableName . "' with more or less than two foreign keys are not supported by the Database Reverese Engineering Driver.");
}
- $this->manyToManyTables[$name] = $table;
+ $this->manyToManyTables[$tableName] = $table;
} else {
- $className = Inflector::classify($name);
- $this->tables[$name] = $table;
- $this->classes[$className] = $name;
+ // lower-casing is necessary because of Oracle Uppercase Tablenames,
+ // assumption is lower-case + underscore separated.
+ $className = Inflector::classify(strtolower($tableName));
+ $this->tables[$tableName] = $table;
+ $this->classToTableNames[$className] = $tableName;
}
}
}
@@ -115,11 +117,11 @@ class DatabaseDriver implements Driver
{
$this->reverseEngineerMappingFromDatabase();
- if (!isset($this->classes[$className])) {
+ if (!isset($this->classToTableNames[$className])) {
throw new \InvalidArgumentException("Unknown class " . $className);
}
- $tableName = Inflector::tableize($className);
+ $tableName = $this->classToTableNames[$className];
$metadata->name = $className;
$metadata->table['name'] = $tableName;
@@ -183,7 +185,7 @@ class DatabaseDriver implements Driver
foreach ($this->manyToManyTables AS $manyTable) {
foreach ($manyTable->getForeignKeys() AS $foreignKey) {
- if ($tableName == strtolower($foreignKey->getForeignTableName())) {
+ if (strtolower($tableName) == strtolower($foreignKey->getForeignTableName())) {
$myFk = $foreignKey;
foreach ($manyTable->getForeignKeys() AS $foreignKey) {
if ($foreignKey != $myFk) {
@@ -269,6 +271,6 @@ class DatabaseDriver implements Driver
{
$this->reverseEngineerMappingFromDatabase();
- return array_keys($this->classes);
+ return array_keys($this->classToTableNames);
}
}
\ No newline at end of file
diff --git a/lib/Doctrine/ORM/Mapping/Driver/DoctrineAnnotations.php b/lib/Doctrine/ORM/Mapping/Driver/DoctrineAnnotations.php
index fb7871c74..435676566 100644
--- a/lib/Doctrine/ORM/Mapping/Driver/DoctrineAnnotations.php
+++ b/lib/Doctrine/ORM/Mapping/Driver/DoctrineAnnotations.php
@@ -114,8 +114,8 @@ final class Index extends Annotation {
final class JoinTable extends Annotation {
public $name;
public $schema;
- public $joinColumns;
- public $inverseJoinColumns;
+ public $joinColumns = array();
+ public $inverseJoinColumns = array();
}
final class SequenceGenerator extends Annotation {
public $sequenceName;
diff --git a/lib/Doctrine/ORM/Mapping/Driver/DriverChain.php b/lib/Doctrine/ORM/Mapping/Driver/DriverChain.php
index d97a61e5b..77c258a18 100644
--- a/lib/Doctrine/ORM/Mapping/Driver/DriverChain.php
+++ b/lib/Doctrine/ORM/Mapping/Driver/DriverChain.php
@@ -88,10 +88,15 @@ class DriverChain implements Driver
public function getAllClassNames()
{
$classNames = array();
- foreach ($this->_drivers AS $driver) {
- $classNames = array_merge($classNames, $driver->getAllClassNames());
+ foreach ($this->_drivers AS $namespace => $driver) {
+ $driverClasses = $driver->getAllClassNames();
+ foreach ($driverClasses AS $className) {
+ if (strpos($className, $namespace) === 0) {
+ $classNames[] = $className;
+ }
+ }
}
- return $classNames;
+ return array_unique($classNames);
}
/**
diff --git a/lib/Doctrine/ORM/Mapping/Driver/StaticPHPDriver.php b/lib/Doctrine/ORM/Mapping/Driver/StaticPHPDriver.php
index 01edab71b..d89b1ed68 100644
--- a/lib/Doctrine/ORM/Mapping/Driver/StaticPHPDriver.php
+++ b/lib/Doctrine/ORM/Mapping/Driver/StaticPHPDriver.php
@@ -78,7 +78,7 @@ class StaticPHPDriver implements Driver
foreach ($this->_paths as $path) {
if ( ! is_dir($path)) {
- throw MappingException::fileMappingDriversRequireConfiguredDirectoryPath();
+ throw MappingException::fileMappingDriversRequireConfiguredDirectoryPath($path);
}
$iterator = new \RecursiveIteratorIterator(
diff --git a/lib/Doctrine/ORM/Mapping/Driver/XmlDriver.php b/lib/Doctrine/ORM/Mapping/Driver/XmlDriver.php
index 7ffcd0001..91892e2b3 100644
--- a/lib/Doctrine/ORM/Mapping/Driver/XmlDriver.php
+++ b/lib/Doctrine/ORM/Mapping/Driver/XmlDriver.php
@@ -87,6 +87,8 @@ class XmlDriver extends AbstractFileDriver
'type' => (string)$discrColumn['type'],
'length' => (string)$discrColumn['length']
));
+ } else {
+ $metadata->setDiscriminatorColumn(array('name' => 'dtype', 'type' => 'string', 'length' => 255));
}
// Evaluate
diff --git a/lib/Doctrine/ORM/Mapping/Driver/YamlDriver.php b/lib/Doctrine/ORM/Mapping/Driver/YamlDriver.php
index 21130c349..e714c50a0 100644
--- a/lib/Doctrine/ORM/Mapping/Driver/YamlDriver.php
+++ b/lib/Doctrine/ORM/Mapping/Driver/YamlDriver.php
@@ -79,6 +79,8 @@ class YamlDriver extends AbstractFileDriver
'type' => $discrColumn['type'],
'length' => $discrColumn['length']
));
+ } else {
+ $metadata->setDiscriminatorColumn(array('name' => 'dtype', 'type' => 'string', 'length' => 255));
}
// Evaluate discriminatorMap
diff --git a/lib/Doctrine/ORM/Mapping/MappingException.php b/lib/Doctrine/ORM/Mapping/MappingException.php
index 9fa5906bf..14bd9ca0b 100644
--- a/lib/Doctrine/ORM/Mapping/MappingException.php
+++ b/lib/Doctrine/ORM/Mapping/MappingException.php
@@ -68,9 +68,9 @@ class MappingException extends \Doctrine\ORM\ORMException
return new self("No mapping file found named '$fileName' for class '$entityName'.");
}
- public static function mappingNotFound($fieldName)
+ public static function mappingNotFound($className, $fieldName)
{
- return new self("No mapping found for field '$fieldName'.");
+ return new self("No mapping found for field '$fieldName' on class '$className'.");
}
public static function oneToManyRequiresMappedBy($fieldName)
@@ -170,9 +170,16 @@ class MappingException extends \Doctrine\ORM\ORMException
);
}
- public static function fileMappingDriversRequireConfiguredDirectoryPath()
+ public static function fileMappingDriversRequireConfiguredDirectoryPath($path = null)
{
- return new self('File mapping drivers must have a valid directory path, however the given path seems to be incorrect!');
+ if ( ! empty($path)) {
+ $path = '[' . $path . ']';
+ }
+
+ return new self(
+ 'File mapping drivers must have a valid directory path, ' .
+ 'however the given path ' . $path . ' seems to be incorrect!'
+ );
}
/**
@@ -200,6 +207,16 @@ class MappingException extends \Doctrine\ORM\ORMException
return new self("Entity class '$className' is using inheritance but no discriminator column was defined.");
}
+ public static function invalidDiscriminatorColumnType($className, $type)
+ {
+ return new self("Discriminator column type on entity class '$className' is not allowed to be '$type'. 'string' or 'integer' type variables are suggested!");
+ }
+
+ public static function cannotVersionIdField($className, $fieldName)
+ {
+ return new self("Setting Id field '$fieldName' as versionale in entity class '$className' is not supported.");
+ }
+
/**
* @param string $className
* @param string $columnName
@@ -209,4 +226,9 @@ class MappingException extends \Doctrine\ORM\ORMException
{
return new self("Duplicate definition of column '".$columnName."' on entity '".$className."' in a field or discriminator column mapping.");
}
+
+ public static function illegalToManyAssocationOnMappedSuperclass($className, $field)
+ {
+ return new self("It is illegal to put a one-to-many or many-to-many association on mapped superclass '".$className."#".$field."'.");
+ }
}
\ No newline at end of file
diff --git a/lib/Doctrine/ORM/ORMException.php b/lib/Doctrine/ORM/ORMException.php
index a4a593470..c84dec41e 100644
--- a/lib/Doctrine/ORM/ORMException.php
+++ b/lib/Doctrine/ORM/ORMException.php
@@ -78,6 +78,14 @@ class ORMException extends Exception
);
}
+ public static function invalidFindByInverseAssociation($entityName, $associationFieldName)
+ {
+ return new self(
+ "You cannot search for the association field '".$entityName."#".$associationFieldName."', ".
+ "because it is the inverse side of an association. Find methods only work on owning side associations."
+ );
+ }
+
public static function invalidResultCacheDriver() {
return new self("Invalid result cache driver; it must implement \Doctrine\Common\Cache\Cache.");
}
diff --git a/lib/Doctrine/ORM/PersistentCollection.php b/lib/Doctrine/ORM/PersistentCollection.php
index 6c09e9697..bf7c6da1e 100644
--- a/lib/Doctrine/ORM/PersistentCollection.php
+++ b/lib/Doctrine/ORM/PersistentCollection.php
@@ -196,7 +196,7 @@ final class PersistentCollection implements Collection
* Initializes the collection by loading its contents from the database
* if the collection is not yet initialized.
*/
- private function initialize()
+ public function initialize()
{
if ( ! $this->initialized && $this->association) {
if ($this->isDirty) {
diff --git a/lib/Doctrine/ORM/Persisters/AbstractEntityInheritancePersister.php b/lib/Doctrine/ORM/Persisters/AbstractEntityInheritancePersister.php
index 9be6de464..bfe1e60d9 100644
--- a/lib/Doctrine/ORM/Persisters/AbstractEntityInheritancePersister.php
+++ b/lib/Doctrine/ORM/Persisters/AbstractEntityInheritancePersister.php
@@ -33,11 +33,18 @@ use Doctrine\ORM\Mapping\ClassMetadata,
abstract class AbstractEntityInheritancePersister extends BasicEntityPersister
{
/**
- * Map from column names to class names that declare the field the column is mapped to.
+ * Map from column names to class metadata instances that declare the field the column is mapped to.
*
* @var array
*/
- private $_declaringClassMap = array();
+ private $declaringClassMap = array();
+
+ /**
+ * Map from column names to class names that declare the field the association with join column is mapped to.
+ *
+ * @var array
+ */
+ private $declaringJoinColumnMap = array();
/**
* {@inheritdoc}
@@ -70,8 +77,8 @@ abstract class AbstractEntityInheritancePersister extends BasicEntityPersister
unset($sqlResult[$discrColumnName]);
foreach ($sqlResult as $column => $value) {
$realColumnName = $this->_resultColumnNames[$column];
- if (isset($this->_declaringClassMap[$column])) {
- $class = $this->_declaringClassMap[$column];
+ if (isset($this->declaringClassMap[$column])) {
+ $class = $this->declaringClassMap[$column];
if ($class->name == $entityName || is_subclass_of($entityName, $class->name)) {
$field = $class->fieldNames[$realColumnName];
if (isset($data[$field])) {
@@ -81,6 +88,10 @@ abstract class AbstractEntityInheritancePersister extends BasicEntityPersister
->convertToPHPValue($value, $this->_platform);
}
}
+ } else if (isset($this->declaringJoinColumnMap[$column])) {
+ if ($this->declaringJoinColumnMap[$column] == $entityName || is_subclass_of($entityName, $this->declaringJoinColumnMap[$column])) {
+ $data[$realColumnName] = $value;
+ }
} else {
$data[$realColumnName] = $value;
}
@@ -99,9 +110,21 @@ abstract class AbstractEntityInheritancePersister extends BasicEntityPersister
$columnAlias = $this->_platform->getSQLResultCasing($columnName . $this->_sqlAliasCounter++);
if ( ! isset($this->_resultColumnNames[$columnAlias])) {
$this->_resultColumnNames[$columnAlias] = $columnName;
- $this->_declaringClassMap[$columnAlias] = $class;
+ $this->declaringClassMap[$columnAlias] = $class;
}
return "$sql AS $columnAlias";
}
+
+ protected function getSelectJoinColumnSQL($tableAlias, $joinColumnName, $className)
+ {
+ $columnAlias = $joinColumnName . $this->_sqlAliasCounter++;
+ $resultColumnName = $this->_platform->getSQLResultCasing($columnAlias);
+ if ( ! isset($this->_resultColumnNames[$resultColumnName])) {
+ $this->_resultColumnNames[$resultColumnName] = $joinColumnName;
+ $this->declaringJoinColumnMap[$resultColumnName] = $className;
+ }
+
+ return $tableAlias . ".$joinColumnName AS $columnAlias";
+ }
}
\ No newline at end of file
diff --git a/lib/Doctrine/ORM/Persisters/JoinedSubclassPersister.php b/lib/Doctrine/ORM/Persisters/JoinedSubclassPersister.php
index 357497bb2..d75656b06 100644
--- a/lib/Doctrine/ORM/Persisters/JoinedSubclassPersister.php
+++ b/lib/Doctrine/ORM/Persisters/JoinedSubclassPersister.php
@@ -35,9 +35,18 @@ class JoinedSubclassPersister extends AbstractEntityInheritancePersister
/**
* Map that maps column names to the table names that own them.
* This is mainly a temporary cache, used during a single request.
+ *
+ * @var array
*/
private $_owningTableMap = array();
+ /**
+ * Map of table to quoted table names.
+ *
+ * @var array
+ */
+ private $_quotedTableMap = array();
+
/**
* {@inheritdoc}
*/
@@ -74,18 +83,16 @@ class JoinedSubclassPersister extends AbstractEntityInheritancePersister
*/
public function getOwningTable($fieldName)
{
- if ( ! isset($this->_owningTableMap[$fieldName])) {
+ if (!isset($this->_owningTableMap[$fieldName])) {
if (isset($this->_class->associationMappings[$fieldName]['inherited'])) {
- $this->_owningTableMap[$fieldName] = $this->_em->getClassMetadata(
- $this->_class->associationMappings[$fieldName]['inherited']
- )->table['name'];
+ $cm = $this->_em->getClassMetadata($this->_class->associationMappings[$fieldName]['inherited']);
} else if (isset($this->_class->fieldMappings[$fieldName]['inherited'])) {
- $this->_owningTableMap[$fieldName] = $this->_em->getClassMetadata(
- $this->_class->fieldMappings[$fieldName]['inherited']
- )->table['name'];
+ $cm = $this->_em->getClassMetadata($this->_class->fieldMappings[$fieldName]['inherited']);
} else {
- $this->_owningTableMap[$fieldName] = $this->_class->table['name'];
+ $cm = $this->_class;
}
+ $this->_owningTableMap[$fieldName] = $cm->table['name'];
+ $this->_quotedTableMap[$cm->table['name']] = $cm->getQuotedTableName($this->_platform);
}
return $this->_owningTableMap[$fieldName];
@@ -186,17 +193,21 @@ class JoinedSubclassPersister extends AbstractEntityInheritancePersister
$updateData = $this->_prepareUpdateData($entity);
if ($isVersioned = $this->_class->isVersioned) {
- $versionedTable = $this->_getVersionedClassMetadata()->table['name'];
+ $versionedClass = $this->_getVersionedClassMetadata();
+ $versionedTable = $versionedClass->table['name'];
}
if ($updateData) {
foreach ($updateData as $tableName => $data) {
- $this->_updateTable($entity, $tableName, $data, $isVersioned && $versionedTable == $tableName);
+ $this->_updateTable($entity, $this->_quotedTableMap[$tableName], $data, $isVersioned && $versionedTable == $tableName);
}
// Make sure the table with the version column is updated even if no columns on that
// table were affected.
if ($isVersioned && ! isset($updateData[$versionedTable])) {
- $this->_updateTable($entity, $versionedTable, array(), true);
+ $this->_updateTable($entity, $versionedClass->getQuotedTableName($this->_platform), array(), true);
+
+ $id = $this->_em->getUnitOfWork()->getEntityIdentifier($entity);
+ $this->_assignDefaultVersionValue($this->_class, $entity, $id);
}
}
}
@@ -252,12 +263,10 @@ class JoinedSubclassPersister extends AbstractEntityInheritancePersister
$this->_getSQLTableAlias($assoc2['inherited'])
: $baseTableAlias;
foreach ($assoc2['targetToSourceKeyColumns'] as $srcColumn) {
- $columnAlias = $srcColumn . $this->_sqlAliasCounter++;
- $columnList .= ", $tableAlias.$srcColumn AS $columnAlias";
- $resultColumnName = $this->_platform->getSQLResultCasing($columnAlias);
- if ( ! isset($this->_resultColumnNames[$resultColumnName])) {
- $this->_resultColumnNames[$resultColumnName] = $srcColumn;
- }
+ if ($columnList != '') $columnList .= ', ';
+ $columnList .= $this->getSelectJoinColumnSQL($tableAlias, $srcColumn,
+ isset($assoc2['inherited']) ? $assoc2['inherited'] : $this->_class->name
+ );
}
}
}
@@ -307,12 +316,10 @@ class JoinedSubclassPersister extends AbstractEntityInheritancePersister
if ($assoc2['isOwningSide'] && $assoc2['type'] & ClassMetadata::TO_ONE
&& ! isset($assoc2['inherited'])) {
foreach ($assoc2['targetToSourceKeyColumns'] as $srcColumn) {
- $columnAlias = $srcColumn . $this->_sqlAliasCounter++;
- $columnList .= ', ' . $tableAlias . ".$srcColumn AS $columnAlias";
- $resultColumnName = $this->_platform->getSQLResultCasing($columnAlias);
- if ( ! isset($this->_resultColumnNames[$resultColumnName])) {
- $this->_resultColumnNames[$resultColumnName] = $srcColumn;
- }
+ if ($columnList != '') $columnList .= ', ';
+ $columnList .= $this->getSelectJoinColumnSQL($tableAlias, $srcColumn,
+ isset($assoc2['inherited']) ? $assoc2['inherited'] : $subClass->name
+ );
}
}
}
@@ -354,6 +361,7 @@ class JoinedSubclassPersister extends AbstractEntityInheritancePersister
*/
public function getLockTablesSql()
{
+ $idColumns = $this->_class->getIdentifierColumnNames();
$baseTableAlias = $this->_getSQLTableAlias($this->_class->name);
// INNER JOIN parent tables
diff --git a/lib/Doctrine/ORM/Persisters/SingleTablePersister.php b/lib/Doctrine/ORM/Persisters/SingleTablePersister.php
index 856ff3485..0ffa93826 100644
--- a/lib/Doctrine/ORM/Persisters/SingleTablePersister.php
+++ b/lib/Doctrine/ORM/Persisters/SingleTablePersister.php
@@ -63,12 +63,10 @@ class SingleTablePersister extends AbstractEntityInheritancePersister
foreach ($subClass->associationMappings as $assoc) {
if ($assoc['isOwningSide'] && $assoc['type'] & ClassMetadata::TO_ONE && ! isset($assoc['inherited'])) {
foreach ($assoc['targetToSourceKeyColumns'] as $srcColumn) {
- $columnAlias = $srcColumn . $this->_sqlAliasCounter++;
- $columnList .= ', ' . $tableAlias . ".$srcColumn AS $columnAlias";
- $resultColumnName = $this->_platform->getSQLResultCasing($columnAlias);
- if ( ! isset($this->_resultColumnNames[$resultColumnName])) {
- $this->_resultColumnNames[$resultColumnName] = $srcColumn;
- }
+ if ($columnList != '') $columnList .= ', ';
+ $columnList .= $this->getSelectJoinColumnSQL($tableAlias, $srcColumn,
+ isset($assoc['inherited']) ? $assoc['inherited'] : $this->_class->name
+ );
}
}
}
diff --git a/lib/Doctrine/ORM/Proxy/ProxyFactory.php b/lib/Doctrine/ORM/Proxy/ProxyFactory.php
index 472e73013..8be75d996 100644
--- a/lib/Doctrine/ORM/Proxy/ProxyFactory.php
+++ b/lib/Doctrine/ORM/Proxy/ProxyFactory.php
@@ -77,9 +77,11 @@ class ProxyFactory
$proxyClassName = str_replace('\\', '', $className) . 'Proxy';
$fqn = $this->_proxyNamespace . '\\' . $proxyClassName;
- if ($this->_autoGenerate && ! class_exists($fqn, false)) {
+ if (! class_exists($fqn, false)) {
$fileName = $this->_proxyDir . DIRECTORY_SEPARATOR . $proxyClassName . '.php';
- $this->_generateProxyClass($this->_em->getClassMetadata($className), $proxyClassName, $fileName, self::$_proxyClassTemplate);
+ if ($this->_autoGenerate) {
+ $this->_generateProxyClass($this->_em->getClassMetadata($className), $proxyClassName, $fileName, self::$_proxyClassTemplate);
+ }
require $fileName;
}
diff --git a/lib/Doctrine/ORM/Query.php b/lib/Doctrine/ORM/Query.php
index 4c755d0f4..d428e2ddb 100644
--- a/lib/Doctrine/ORM/Query.php
+++ b/lib/Doctrine/ORM/Query.php
@@ -242,10 +242,14 @@ final class Query extends AbstractQuery
}
if (is_object($value) && $this->_em->getMetadataFactory()->hasMetadataFor(get_class($value))) {
- //TODO: Check that $value is MANAGED?
- $values = $this->_em->getUnitOfWork()->getEntityIdentifier($value);
+ if ($this->_em->getUnitOfWork()->getEntityState($value) == UnitOfWork::STATE_MANAGED) {
+ $idValues = $this->_em->getUnitOfWork()->getEntityIdentifier($value);
+ } else {
+ $class = $this->_em->getClassMetadata(get_class($value));
+ $idValues = $class->getIdentifierValues($value);
+ }
$sqlPositions = $paramMappings[$key];
- $sqlParams += array_combine((array)$sqlPositions, $values);
+ $sqlParams += array_combine((array)$sqlPositions, $idValues);
} else {
foreach ($paramMappings[$key] as $position) {
$sqlParams[$position] = $value;
@@ -544,4 +548,15 @@ final class Query extends AbstractQuery
'&hydrationMode='.$this->_hydrationMode.'DOCTRINE_QUERY_CACHE_SALT'
);
}
+
+ /**
+ * Cleanup Query resource when clone is called.
+ *
+ * @return void
+ */
+ public function __clone()
+ {
+ parent::__clone();
+ $this->_state = self::STATE_DIRTY;
+ }
}
\ No newline at end of file
diff --git a/lib/Doctrine/ORM/Query/AST/Functions/LengthFunction.php b/lib/Doctrine/ORM/Query/AST/Functions/LengthFunction.php
index fc779e4f7..36787786d 100644
--- a/lib/Doctrine/ORM/Query/AST/Functions/LengthFunction.php
+++ b/lib/Doctrine/ORM/Query/AST/Functions/LengthFunction.php
@@ -41,8 +41,9 @@ class LengthFunction extends FunctionNode
*/
public function getSql(\Doctrine\ORM\Query\SqlWalker $sqlWalker)
{
- //TODO: Use platform to get SQL
- return 'LENGTH(' . $sqlWalker->walkStringPrimary($this->stringPrimary) . ')';
+ return $sqlWalker->getConnection()->getDatabasePlatform()->getLengthExpression(
+ $sqlWalker->walkSimpleArithmeticExpression($this->stringPrimary)
+ );
}
/**
diff --git a/lib/Doctrine/ORM/Query/AST/Functions/LowerFunction.php b/lib/Doctrine/ORM/Query/AST/Functions/LowerFunction.php
index b5d33face..775f51d9a 100644
--- a/lib/Doctrine/ORM/Query/AST/Functions/LowerFunction.php
+++ b/lib/Doctrine/ORM/Query/AST/Functions/LowerFunction.php
@@ -41,8 +41,9 @@ class LowerFunction extends FunctionNode
*/
public function getSql(\Doctrine\ORM\Query\SqlWalker $sqlWalker)
{
- //TODO: Use platform to get SQL
- return 'LOWER(' . $sqlWalker->walkStringPrimary($this->stringPrimary) . ')';
+ return $sqlWalker->getConnection()->getDatabasePlatform()->getLowerExpression(
+ $sqlWalker->walkSimpleArithmeticExpression($this->stringPrimary)
+ );
}
/**
diff --git a/lib/Doctrine/ORM/Query/AST/Functions/ModFunction.php b/lib/Doctrine/ORM/Query/AST/Functions/ModFunction.php
index 7c52d60b8..4d124fe85 100644
--- a/lib/Doctrine/ORM/Query/AST/Functions/ModFunction.php
+++ b/lib/Doctrine/ORM/Query/AST/Functions/ModFunction.php
@@ -42,12 +42,10 @@ class ModFunction extends FunctionNode
*/
public function getSql(\Doctrine\ORM\Query\SqlWalker $sqlWalker)
{
- //TODO: Use platform to get SQL
- return 'MOD('
- . $sqlWalker->walkSimpleArithmeticExpression($this->firstSimpleArithmeticExpression)
- . ', '
- . $sqlWalker->walkSimpleArithmeticExpression($this->secondSimpleArithmeticExpression)
- . ')';
+ return $sqlWalker->getConnection()->getDatabasePlatform()->getModExpression(
+ $sqlWalker->walkSimpleArithmeticExpression($this->firstSimpleArithmeticExpression),
+ $sqlWalker->walkSimpleArithmeticExpression($this->secondSimpleArithmeticExpression)
+ );
}
/**
diff --git a/lib/Doctrine/ORM/Query/AST/Functions/UpperFunction.php b/lib/Doctrine/ORM/Query/AST/Functions/UpperFunction.php
index ff2b1e30b..acc8dd8eb 100644
--- a/lib/Doctrine/ORM/Query/AST/Functions/UpperFunction.php
+++ b/lib/Doctrine/ORM/Query/AST/Functions/UpperFunction.php
@@ -41,8 +41,9 @@ class UpperFunction extends FunctionNode
*/
public function getSql(\Doctrine\ORM\Query\SqlWalker $sqlWalker)
{
- //TODO: Use platform to get SQL
- return 'UPPER(' . $sqlWalker->walkStringPrimary($this->stringPrimary) . ')';
+ return $sqlWalker->getConnection()->getDatabasePlatform()->getUpperExpression(
+ $sqlWalker->walkSimpleArithmeticExpression($this->stringPrimary)
+ );
}
/**
diff --git a/lib/Doctrine/ORM/Query/Parser.php b/lib/Doctrine/ORM/Query/Parser.php
index 3d3fb5f5e..0c22c9fbf 100644
--- a/lib/Doctrine/ORM/Query/Parser.php
+++ b/lib/Doctrine/ORM/Query/Parser.php
@@ -125,6 +125,11 @@ class Parser
*/
private $_customOutputWalker;
+ /**
+ * @var array
+ */
+ private $_identVariableExpressions = array();
+
/**
* Creates a new query parser object.
*
@@ -272,6 +277,9 @@ class Parser
{
$AST = $this->getAST();
+ $this->fixIdentificationVariableOrder($AST);
+ $this->assertSelectEntityRootAliasRequirement();
+
if (($customWalkers = $this->_query->getHint(Query::HINT_CUSTOM_TREE_WALKERS)) !== false) {
$this->_customTreeWalkers = $customWalkers;
}
@@ -312,6 +320,46 @@ class Parser
return $this->_parserResult;
}
+
+ private function assertSelectEntityRootAliasRequirement()
+ {
+ if ( count($this->_identVariableExpressions) > 0) {
+ $foundRootEntity = false;
+ foreach ($this->_identVariableExpressions AS $dqlAlias => $expr) {
+ if (isset($this->_queryComponents[$dqlAlias]) && $this->_queryComponents[$dqlAlias]['parent'] === null) {
+ $foundRootEntity = true;
+ }
+ }
+
+ if (!$foundRootEntity) {
+ $this->semanticalError('Cannot select entity through identification variables without choosing at least one root entity alias.');
+ }
+ }
+ }
+
+ /**
+ * Fix order of identification variables.
+ *
+ * They have to appear in the select clause in the same order as the
+ * declarations (from ... x join ... y join ... z ...) appear in the query
+ * as the hydration process relies on that order for proper operation.
+ *
+ * @param AST\SelectStatement|AST\DeleteStatement|AST\UpdateStatement $AST
+ * @return void
+ */
+ private function fixIdentificationVariableOrder($AST)
+ {
+ if ( count($this->_identVariableExpressions) > 1) {
+ foreach ($this->_queryComponents as $dqlAlias => $qComp) {
+ if (isset($this->_identVariableExpressions[$dqlAlias])) {
+ $expr = $this->_identVariableExpressions[$dqlAlias];
+ $key = array_search($expr, $AST->selectClause->selectExpressions);
+ unset($AST->selectClause->selectExpressions[$key]);
+ $AST->selectClause->selectExpressions[] = $expr;
+ }
+ }
+ }
+ }
/**
* Generates a new syntax error.
@@ -1628,6 +1676,7 @@ class Parser
public function SelectExpression()
{
$expression = null;
+ $identVariable = null;
$fieldAliasIdentificationVariable = null;
$peek = $this->_lexer->glimpse();
@@ -1639,7 +1688,7 @@ class Parser
$expression = $this->ScalarExpression();
} else {
$supportsAlias = false;
- $expression = $this->IdentificationVariable();
+ $expression = $identVariable = $this->IdentificationVariable();
}
} else if ($this->_lexer->lookahead['value'] == '(') {
if ($peek['type'] == Lexer::T_SELECT) {
@@ -1666,6 +1715,7 @@ class Parser
} else if ($this->_lexer->lookahead['type'] == Lexer::T_PARTIAL) {
$supportsAlias = false;
$expression = $this->PartialObjectExpression();
+ $identVariable = $expression->identificationVariable;
} else if ($this->_lexer->lookahead['type'] == Lexer::T_INTEGER ||
$this->_lexer->lookahead['type'] == Lexer::T_FLOAT) {
// Shortcut: ScalarExpression => SimpleArithmeticExpression
@@ -1694,7 +1744,11 @@ class Parser
}
}
- return new AST\SelectExpression($expression, $fieldAliasIdentificationVariable);
+ $expr = new AST\SelectExpression($expression, $fieldAliasIdentificationVariable);
+ if (!$supportsAlias) {
+ $this->_identVariableExpressions[$identVariable] = $expr;
+ }
+ return $expr;
}
/**
diff --git a/lib/Doctrine/ORM/Query/SqlWalker.php b/lib/Doctrine/ORM/Query/SqlWalker.php
index 77fe7bc02..1512da236 100644
--- a/lib/Doctrine/ORM/Query/SqlWalker.php
+++ b/lib/Doctrine/ORM/Query/SqlWalker.php
@@ -689,7 +689,7 @@ class SqlWalker implements TreeWalker
$sql = $this->_scalarResultAliasMap[$columnName];
}
- return $sql . ' ' . strtoupper($orderByItem->type);;
+ return $sql . ' ' . strtoupper($orderByItem->type);
}
/**
diff --git a/lib/Doctrine/ORM/QueryBuilder.php b/lib/Doctrine/ORM/QueryBuilder.php
index bcea7c000..436522b69 100644
--- a/lib/Doctrine/ORM/QueryBuilder.php
+++ b/lib/Doctrine/ORM/QueryBuilder.php
@@ -888,6 +888,40 @@ class QueryBuilder
. (isset($options['post']) ? $options['post'] : '');
}
+ /**
+ * Reset DQL parts
+ *
+ * @param array $parts
+ * @return QueryBuilder
+ */
+ public function resetDQLParts($parts = null)
+ {
+ if (is_null($parts)) {
+ $parts = array_keys($this->_dqlParts);
+ }
+ foreach ($parts as $part) {
+ $this->resetDQLPart($part);
+ }
+ return $this;
+ }
+
+ /**
+ * Reset single DQL part
+ *
+ * @param string $part
+ * @return QueryBuilder;
+ */
+ public function resetDQLPart($part)
+ {
+ if (is_array($this->_dqlParts[$part])) {
+ $this->_dqlParts[$part] = array();
+ } else {
+ $this->_dqlParts[$part] = null;
+ }
+ $this->_state = self::STATE_DIRTY;
+ return $this;
+ }
+
/**
* Gets a string representation of this QueryBuilder which corresponds to
* the final DQL query being constructed.
@@ -898,4 +932,24 @@ class QueryBuilder
{
return $this->getDQL();
}
+
+ /**
+ * Deep clone of all expression objects in the DQL parts.
+ *
+ * @return void
+ */
+ public function __clone()
+ {
+ foreach ($this->_dqlParts AS $part => $elements) {
+ if (is_array($this->_dqlParts[$part])) {
+ foreach ($this->_dqlParts[$part] AS $idx => $element) {
+ if (is_object($element)) {
+ $this->_dqlParts[$part][$idx] = clone $element;
+ }
+ }
+ } else if (\is_object($elements)) {
+ $this->_dqlParts[$part] = clone $elements;
+ }
+ }
+ }
}
\ No newline at end of file
diff --git a/lib/Doctrine/ORM/Tools/Console/Command/ClearCache/ResultCommand.php b/lib/Doctrine/ORM/Tools/Console/Command/ClearCache/ResultCommand.php
index d994d708f..f1506809b 100644
--- a/lib/Doctrine/ORM/Tools/Console/Command/ClearCache/ResultCommand.php
+++ b/lib/Doctrine/ORM/Tools/Console/Command/ClearCache/ResultCommand.php
@@ -49,19 +49,19 @@ class ResultCommand extends Console\Command\Command
->setDescription('Clear result cache of the various cache drivers.')
->setDefinition(array(
new InputOption(
- 'id', null, InputOption::PARAMETER_REQUIRED | InputOption::PARAMETER_IS_ARRAY,
+ 'id', null, InputOption::VALUE_REQUIRED | InputOption::VALUE_IS_ARRAY,
'ID(s) of the cache entry to delete (accepts * wildcards).', array()
),
new InputOption(
- 'regex', null, InputOption::PARAMETER_REQUIRED | InputOption::PARAMETER_IS_ARRAY,
+ 'regex', null, InputOption::VALUE_REQUIRED | InputOption::VALUE_IS_ARRAY,
'Delete cache entries that match the given regular expression(s).', array()
),
new InputOption(
- 'prefix', null, InputOption::PARAMETER_REQUIRED | InputOption::PARAMETER_IS_ARRAY,
+ 'prefix', null, InputOption::VALUE_REQUIRED | InputOption::VALUE_IS_ARRAY,
'Delete cache entries that have the given prefix(es).', array()
),
new InputOption(
- 'suffix', null, InputOption::PARAMETER_REQUIRED | InputOption::PARAMETER_IS_ARRAY,
+ 'suffix', null, InputOption::VALUE_REQUIRED | InputOption::VALUE_IS_ARRAY,
'Delete cache entries that have the given suffix(es).', array()
),
))
diff --git a/lib/Doctrine/ORM/Tools/Console/Command/ConvertDoctrine1SchemaCommand.php b/lib/Doctrine/ORM/Tools/Console/Command/ConvertDoctrine1SchemaCommand.php
index a518b76e8..1a1328aac 100644
--- a/lib/Doctrine/ORM/Tools/Console/Command/ConvertDoctrine1SchemaCommand.php
+++ b/lib/Doctrine/ORM/Tools/Console/Command/ConvertDoctrine1SchemaCommand.php
@@ -112,16 +112,16 @@ class ConvertDoctrine1SchemaCommand extends Console\Command\Command
'The path to generate your Doctrine 2.X mapping information.'
),
new InputOption(
- 'from', null, InputOption::PARAMETER_REQUIRED | InputOption::PARAMETER_IS_ARRAY,
+ 'from', null, InputOption::VALUE_REQUIRED | InputOption::VALUE_IS_ARRAY,
'Optional paths of Doctrine 1.X schema information.',
array()
),
new InputOption(
- 'extend', null, InputOption::PARAMETER_OPTIONAL,
+ 'extend', null, InputOption::VALUE_OPTIONAL,
'Defines a base class to be extended by generated entity classes.'
),
new InputOption(
- 'num-spaces', null, InputOption::PARAMETER_OPTIONAL,
+ 'num-spaces', null, InputOption::VALUE_OPTIONAL,
'Defines the number of indentation spaces', 4
)
))
diff --git a/lib/Doctrine/ORM/Tools/Console/Command/ConvertMappingCommand.php b/lib/Doctrine/ORM/Tools/Console/Command/ConvertMappingCommand.php
index ad3134779..797bc29a8 100644
--- a/lib/Doctrine/ORM/Tools/Console/Command/ConvertMappingCommand.php
+++ b/lib/Doctrine/ORM/Tools/Console/Command/ConvertMappingCommand.php
@@ -53,7 +53,7 @@ class ConvertMappingCommand extends Console\Command\Command
->setDescription('Convert mapping information between supported formats.')
->setDefinition(array(
new InputOption(
- 'filter', null, InputOption::PARAMETER_REQUIRED | InputOption::PARAMETER_IS_ARRAY,
+ 'filter', null, InputOption::VALUE_REQUIRED | InputOption::VALUE_IS_ARRAY,
'A string pattern used to match entities that should be processed.'
),
new InputArgument(
@@ -67,11 +67,11 @@ class ConvertMappingCommand extends Console\Command\Command
'from-database', null, null, 'Whether or not to convert mapping information from existing database.'
),
new InputOption(
- 'extend', null, InputOption::PARAMETER_OPTIONAL,
+ 'extend', null, InputOption::VALUE_OPTIONAL,
'Defines a base class to be extended by generated entity classes.'
),
new InputOption(
- 'num-spaces', null, InputOption::PARAMETER_OPTIONAL,
+ 'num-spaces', null, InputOption::VALUE_OPTIONAL,
'Defines the number of indentation spaces', 4
)
))
@@ -96,7 +96,8 @@ EOT
);
}
- $cmf = new DisconnectedClassMetadataFactory($em);
+ $cmf = new DisconnectedClassMetadataFactory();
+ $cmf->setEntityManager($em);
$metadata = $cmf->getAllMetadata();
$metadata = MetadataFilter::filter($metadata, $input->getOption('filter'));
diff --git a/lib/Doctrine/ORM/Tools/Console/Command/EnsureProductionSettingsCommand.php b/lib/Doctrine/ORM/Tools/Console/Command/EnsureProductionSettingsCommand.php
index 4fdd8fad7..d29bfce0b 100644
--- a/lib/Doctrine/ORM/Tools/Console/Command/EnsureProductionSettingsCommand.php
+++ b/lib/Doctrine/ORM/Tools/Console/Command/EnsureProductionSettingsCommand.php
@@ -49,7 +49,7 @@ class EnsureProductionSettingsCommand extends Console\Command\Command
->setDescription('Verify that Doctrine is properly configured for a production environment.')
->setDefinition(array(
new InputOption(
- 'complete', null, InputOption::PARAMETER_NONE,
+ 'complete', null, InputOption::VALUE_NONE,
'Flag to also inspect database connection existance.'
)
))
diff --git a/lib/Doctrine/ORM/Tools/Console/Command/GenerateEntitiesCommand.php b/lib/Doctrine/ORM/Tools/Console/Command/GenerateEntitiesCommand.php
index 08654ba18..f69b5167f 100644
--- a/lib/Doctrine/ORM/Tools/Console/Command/GenerateEntitiesCommand.php
+++ b/lib/Doctrine/ORM/Tools/Console/Command/GenerateEntitiesCommand.php
@@ -52,34 +52,34 @@ class GenerateEntitiesCommand extends Console\Command\Command
->setDescription('Generate entity classes and method stubs from your mapping information.')
->setDefinition(array(
new InputOption(
- 'filter', null, InputOption::PARAMETER_REQUIRED | InputOption::PARAMETER_IS_ARRAY,
+ 'filter', null, InputOption::VALUE_REQUIRED | InputOption::VALUE_IS_ARRAY,
'A string pattern used to match entities that should be processed.'
),
new InputArgument(
'dest-path', InputArgument::REQUIRED, 'The path to generate your entity classes.'
),
new InputOption(
- 'generate-annotations', null, InputOption::PARAMETER_OPTIONAL,
+ 'generate-annotations', null, InputOption::VALUE_OPTIONAL,
'Flag to define if generator should generate annotation metadata on entities.', false
),
new InputOption(
- 'generate-methods', null, InputOption::PARAMETER_OPTIONAL,
+ 'generate-methods', null, InputOption::VALUE_OPTIONAL,
'Flag to define if generator should generate stub methods on entities.', true
),
new InputOption(
- 'regenerate-entities', null, InputOption::PARAMETER_OPTIONAL,
+ 'regenerate-entities', null, InputOption::VALUE_OPTIONAL,
'Flag to define if generator should regenerate entity if it exists.', false
),
new InputOption(
- 'update-entities', null, InputOption::PARAMETER_OPTIONAL,
+ 'update-entities', null, InputOption::VALUE_OPTIONAL,
'Flag to define if generator should only update entity if it exists.', true
),
new InputOption(
- 'extend', null, InputOption::PARAMETER_OPTIONAL,
+ 'extend', null, InputOption::VALUE_OPTIONAL,
'Defines a base class to be extended by generated entity classes.'
),
new InputOption(
- 'num-spaces', null, InputOption::PARAMETER_OPTIONAL,
+ 'num-spaces', null, InputOption::VALUE_OPTIONAL,
'Defines the number of indentation spaces', 4
)
))
@@ -96,7 +96,8 @@ EOT
{
$em = $this->getHelper('em')->getEntityManager();
- $cmf = new DisconnectedClassMetadataFactory($em);
+ $cmf = new DisconnectedClassMetadataFactory();
+ $cmf->setEntityManager($em);
$metadatas = $cmf->getAllMetadata();
$metadatas = MetadataFilter::filter($metadatas, $input->getOption('filter'));
diff --git a/lib/Doctrine/ORM/Tools/Console/Command/GenerateProxiesCommand.php b/lib/Doctrine/ORM/Tools/Console/Command/GenerateProxiesCommand.php
index 09376d867..987628956 100644
--- a/lib/Doctrine/ORM/Tools/Console/Command/GenerateProxiesCommand.php
+++ b/lib/Doctrine/ORM/Tools/Console/Command/GenerateProxiesCommand.php
@@ -50,7 +50,7 @@ class GenerateProxiesCommand extends Console\Command\Command
->setDescription('Generates proxy classes for entity classes.')
->setDefinition(array(
new InputOption(
- 'filter', null, InputOption::PARAMETER_REQUIRED | InputOption::PARAMETER_IS_ARRAY,
+ 'filter', null, InputOption::VALUE_REQUIRED | InputOption::VALUE_IS_ARRAY,
'A string pattern used to match entities that should be processed.'
),
new InputArgument(
diff --git a/lib/Doctrine/ORM/Tools/Console/Command/GenerateRepositoriesCommand.php b/lib/Doctrine/ORM/Tools/Console/Command/GenerateRepositoriesCommand.php
index a70aa39d0..30d36eb44 100644
--- a/lib/Doctrine/ORM/Tools/Console/Command/GenerateRepositoriesCommand.php
+++ b/lib/Doctrine/ORM/Tools/Console/Command/GenerateRepositoriesCommand.php
@@ -51,7 +51,7 @@ class GenerateRepositoriesCommand extends Console\Command\Command
->setDescription('Generate repository classes from your mapping information.')
->setDefinition(array(
new InputOption(
- 'filter', null, InputOption::PARAMETER_REQUIRED | InputOption::PARAMETER_IS_ARRAY,
+ 'filter', null, InputOption::VALUE_REQUIRED | InputOption::VALUE_IS_ARRAY,
'A string pattern used to match entities that should be processed.'
),
new InputArgument(
diff --git a/lib/Doctrine/ORM/Tools/Console/Command/RunDqlCommand.php b/lib/Doctrine/ORM/Tools/Console/Command/RunDqlCommand.php
index 678b045d3..c794cb996 100644
--- a/lib/Doctrine/ORM/Tools/Console/Command/RunDqlCommand.php
+++ b/lib/Doctrine/ORM/Tools/Console/Command/RunDqlCommand.php
@@ -50,20 +50,20 @@ class RunDqlCommand extends Console\Command\Command
->setDefinition(array(
new InputArgument('dql', InputArgument::REQUIRED, 'The DQL to execute.'),
new InputOption(
- 'hydrate', null, InputOption::PARAMETER_REQUIRED,
+ 'hydrate', null, InputOption::VALUE_REQUIRED,
'Hydration mode of result set. Should be either: object, array, scalar or single-scalar.',
'object'
),
new InputOption(
- 'first-result', null, InputOption::PARAMETER_REQUIRED,
+ 'first-result', null, InputOption::VALUE_REQUIRED,
'The first result in the result set.'
),
new InputOption(
- 'max-result', null, InputOption::PARAMETER_REQUIRED,
+ 'max-result', null, InputOption::VALUE_REQUIRED,
'The maximum number of results in the result set.'
),
new InputOption(
- 'depth', null, InputOption::PARAMETER_REQUIRED,
+ 'depth', null, InputOption::VALUE_REQUIRED,
'Dumping depth of Entity graph.', 7
)
))
@@ -114,7 +114,7 @@ EOT
throw new \LogicException("Option 'max-result' must contains an integer value");
}
- $query->setMaxResult((int) $maxResult);
+ $query->setMaxResults((int) $maxResult);
}
$resultSet = $query->execute(array(), constant($hydrationMode));
diff --git a/lib/Doctrine/ORM/Tools/Console/Command/SchemaTool/CreateCommand.php b/lib/Doctrine/ORM/Tools/Console/Command/SchemaTool/CreateCommand.php
index c3870221b..e18a9c56a 100644
--- a/lib/Doctrine/ORM/Tools/Console/Command/SchemaTool/CreateCommand.php
+++ b/lib/Doctrine/ORM/Tools/Console/Command/SchemaTool/CreateCommand.php
@@ -53,7 +53,7 @@ class CreateCommand extends AbstractCommand
)
->setDefinition(array(
new InputOption(
- 'dump-sql', null, InputOption::PARAMETER_NONE,
+ 'dump-sql', null, InputOption::VALUE_NONE,
'Instead of try to apply generated SQLs into EntityManager Storage Connection, output them.'
)
))
@@ -65,6 +65,8 @@ EOT
protected function executeSchemaCommand(InputInterface $input, OutputInterface $output, SchemaTool $schemaTool, array $metadatas)
{
+ $output->write('ATTENTION: This operation should not be executed in an production enviroment.' . PHP_EOL . PHP_EOL);
+
if ($input->getOption('dump-sql') === true) {
$sqls = $schemaTool->getCreateSchemaSql($metadatas);
$output->write(implode(';' . PHP_EOL, $sqls) . PHP_EOL);
diff --git a/lib/Doctrine/ORM/Tools/Console/Command/SchemaTool/DropCommand.php b/lib/Doctrine/ORM/Tools/Console/Command/SchemaTool/DropCommand.php
index 0d6741717..82d91f4c6 100644
--- a/lib/Doctrine/ORM/Tools/Console/Command/SchemaTool/DropCommand.php
+++ b/lib/Doctrine/ORM/Tools/Console/Command/SchemaTool/DropCommand.php
@@ -49,13 +49,21 @@ class DropCommand extends AbstractCommand
$this
->setName('orm:schema-tool:drop')
->setDescription(
- 'Processes the schema and either drop the database schema of EntityManager Storage Connection or generate the SQL output.'
+ 'Drop the complete database schema of EntityManager Storage Connection or generate the corresponding SQL output.'
)
->setDefinition(array(
new InputOption(
- 'dump-sql', null, InputOption::PARAMETER_NONE,
+ 'dump-sql', null, InputOption::VALUE_NONE,
'Instead of try to apply generated SQLs into EntityManager Storage Connection, output them.'
- )
+ ),
+ new InputOption(
+ 'force', null, InputOption::VALUE_NONE,
+ "Don't ask for the deletion of the database, but force the operation to run."
+ ),
+ new InputOption(
+ 'full-database', null, InputOption::VALUE_NONE,
+ 'Instead of using the Class Metadata to detect the database table schema, drop ALL assets that the database contains.'
+ ),
))
->setHelp(<<getOption('full-database'));
+
if ($input->getOption('dump-sql') === true) {
- $sqls = $schemaTool->getDropSchemaSql($metadatas);
+ if ($isFullDatabaseDrop) {
+ $sqls = $schemaTool->getDropDatabaseSQL();
+ } else {
+ $sqls = $schemaTool->getDropSchemaSQL($metadatas);
+ }
$output->write(implode(';' . PHP_EOL, $sqls) . PHP_EOL);
- } else {
+ } else if ($input->getOption('force') === true) {
$output->write('Dropping database schema...' . PHP_EOL);
- $schemaTool->dropSchema($metadatas);
+ if ($isFullDatabaseDrop) {
+ $schemaTool->dropDatabase();
+ } else {
+ $schemaTool->dropSchema($metadatas);
+ }
$output->write('Database schema dropped successfully!' . PHP_EOL);
+ } else {
+ $output->write('ATTENTION: This operation should not be executed in an production enviroment.' . PHP_EOL . PHP_EOL);
+
+ if ($isFullDatabaseDrop) {
+ $sqls = $schemaTool->getDropDatabaseSQL();
+ } else {
+ $sqls = $schemaTool->getDropSchemaSQL($metadatas);
+ }
+
+ if (count($sqls)) {
+ $output->write('Schema-Tool would execute ' . count($sqls) . ' queries to drop the database.' . PHP_EOL);
+ $output->write('Please run the operation with --force to execute these queries or use --dump-sql to see them.' . PHP_EOL);
+ } else {
+ $output->write('Nothing to drop. The database is empty!' . PHP_EOL);
+ }
}
}
}
diff --git a/lib/Doctrine/ORM/Tools/Console/Command/SchemaTool/UpdateCommand.php b/lib/Doctrine/ORM/Tools/Console/Command/SchemaTool/UpdateCommand.php
index 986d97954..f1a3a73cf 100644
--- a/lib/Doctrine/ORM/Tools/Console/Command/SchemaTool/UpdateCommand.php
+++ b/lib/Doctrine/ORM/Tools/Console/Command/SchemaTool/UpdateCommand.php
@@ -53,13 +53,17 @@ class UpdateCommand extends AbstractCommand
)
->setDefinition(array(
new InputOption(
- 'complete', null, InputOption::PARAMETER_NONE,
+ 'complete', null, InputOption::VALUE_NONE,
'If defined, all assets of the database which are not relevant to the current metadata will be dropped.'
),
new InputOption(
- 'dump-sql', null, InputOption::PARAMETER_NONE,
+ 'dump-sql', null, InputOption::VALUE_NONE,
'Instead of try to apply generated SQLs into EntityManager Storage Connection, output them.'
- )
+ ),
+ new InputOption(
+ 'force', null, InputOption::VALUE_NONE,
+ "Don't ask for the incremental update of the database, but force the operation to run."
+ ),
))
->setHelp(<<getOption('complete') === true);
+ $saveMode = ($input->getOption('complete') !== true);
if ($input->getOption('dump-sql') === true) {
$sqls = $schemaTool->getUpdateSchemaSql($metadatas, $saveMode);
$output->write(implode(';' . PHP_EOL, $sqls) . PHP_EOL);
- } else {
+ } else if ($input->getOption('force') === true) {
$output->write('Updating database schema...' . PHP_EOL);
$schemaTool->updateSchema($metadatas, $saveMode);
$output->write('Database schema updated successfully!' . PHP_EOL);
+ } else {
+ $output->write('ATTENTION: This operation should not be executed in an production enviroment.' . PHP_EOL);
+ $output->write('Use the incremental update to detect changes during development and use' . PHP_EOL);
+ $output->write('this SQL DDL to manually update your database in production.' . PHP_EOL . PHP_EOL);
+
+ $sqls = $schemaTool->getUpdateSchemaSql($metadatas, $saveMode);
+
+ if (count($sqls)) {
+ $output->write('Schema-Tool would execute ' . count($sqls) . ' queries to update the database.' . PHP_EOL);
+ $output->write('Please run the operation with --force to execute these queries or use --dump-sql to see them.' . PHP_EOL);
+ } else {
+ $output->write('Nothing to update. The database is in sync with the current entity metadata.' . PHP_EOL);
+ }
}
}
}
diff --git a/lib/Doctrine/ORM/Tools/Console/ConsoleRunner.php b/lib/Doctrine/ORM/Tools/Console/ConsoleRunner.php
new file mode 100644
index 000000000..f74713a7c
--- /dev/null
+++ b/lib/Doctrine/ORM/Tools/Console/ConsoleRunner.php
@@ -0,0 +1,69 @@
+.
+ */
+
+namespace Doctrine\ORM\Tools\Console;
+
+use Symfony\Component\Console\Application;
+use Symfony\Component\Console\Helper\HelperSet;
+
+class ConsoleRunner
+{
+ /**
+ * Run console with the given helperset.
+ *
+ * @param \Symfony\Component\Console\Helper\HelperSet $helperSet
+ * @return void
+ */
+ static public function run(HelperSet $helperSet)
+ {
+ $cli = new Application('Doctrine Command Line Interface', \Doctrine\ORM\Version::VERSION);
+ $cli->setCatchExceptions(true);
+ $cli->setHelperSet($helperSet);
+ self::addCommands($cli);
+ $cli->run();
+ }
+
+ /**
+ * @param Application $cli
+ */
+ static public function addCommands(Application $cli)
+ {
+ $cli->addCommands(array(
+ // DBAL Commands
+ new \Doctrine\DBAL\Tools\Console\Command\RunSqlCommand(),
+ new \Doctrine\DBAL\Tools\Console\Command\ImportCommand(),
+
+ // ORM Commands
+ new \Doctrine\ORM\Tools\Console\Command\ClearCache\MetadataCommand(),
+ new \Doctrine\ORM\Tools\Console\Command\ClearCache\ResultCommand(),
+ new \Doctrine\ORM\Tools\Console\Command\ClearCache\QueryCommand(),
+ new \Doctrine\ORM\Tools\Console\Command\SchemaTool\CreateCommand(),
+ new \Doctrine\ORM\Tools\Console\Command\SchemaTool\UpdateCommand(),
+ new \Doctrine\ORM\Tools\Console\Command\SchemaTool\DropCommand(),
+ new \Doctrine\ORM\Tools\Console\Command\EnsureProductionSettingsCommand(),
+ new \Doctrine\ORM\Tools\Console\Command\ConvertDoctrine1SchemaCommand(),
+ new \Doctrine\ORM\Tools\Console\Command\GenerateRepositoriesCommand(),
+ new \Doctrine\ORM\Tools\Console\Command\GenerateEntitiesCommand(),
+ new \Doctrine\ORM\Tools\Console\Command\GenerateProxiesCommand(),
+ new \Doctrine\ORM\Tools\Console\Command\ConvertMappingCommand(),
+ new \Doctrine\ORM\Tools\Console\Command\RunDqlCommand(),
+ new \Doctrine\ORM\Tools\Console\Command\ValidateSchemaCommand(),
+ ));
+ }
+}
\ No newline at end of file
diff --git a/lib/Doctrine/ORM/Tools/ConvertDoctrine1Schema.php b/lib/Doctrine/ORM/Tools/ConvertDoctrine1Schema.php
index 14c9ada9a..ecd04ce6d 100644
--- a/lib/Doctrine/ORM/Tools/ConvertDoctrine1Schema.php
+++ b/lib/Doctrine/ORM/Tools/ConvertDoctrine1Schema.php
@@ -188,7 +188,6 @@ class ConvertDoctrine1Schema
$metadata->setIdGeneratorType(ClassMetadataInfo::GENERATOR_TYPE_AUTO);
} else if (isset($column['sequence'])) {
$metadata->setIdGeneratorType(ClassMetadataInfo::GENERATOR_TYPE_SEQUENCE);
- $metadata->setSequenceGeneratorDefinition($definition);
$definition = array(
'sequenceName' => is_array($column['sequence']) ? $column['sequence']['name']:$column['sequence']
);
@@ -198,6 +197,7 @@ class ConvertDoctrine1Schema
if (isset($column['sequence']['value'])) {
$definition['initialValue'] = $column['sequence']['value'];
}
+ $metadata->setSequenceGeneratorDefinition($definition);
}
return $fieldMapping;
}
diff --git a/lib/Doctrine/ORM/Tools/DisconnectedClassMetadataFactory.php b/lib/Doctrine/ORM/Tools/DisconnectedClassMetadataFactory.php
index 436c04b4f..55503d400 100644
--- a/lib/Doctrine/ORM/Tools/DisconnectedClassMetadataFactory.php
+++ b/lib/Doctrine/ORM/Tools/DisconnectedClassMetadataFactory.php
@@ -1,7 +1,5 @@
* @author Guilherme Blanco
* @author Jonathan Wage
@@ -46,7 +43,13 @@ class DisconnectedClassMetadataFactory extends ClassMetadataFactory
*/
protected function newClassMetadataInstance($className)
{
- return new ClassMetadataInfo($className);
+ $metadata = new ClassMetadataInfo($className);
+ if (strpos($className, "\\") !== false) {
+ $metadata->namespace = strrev(substr( strrev($className), strpos(strrev($className), "\\")+1 ));
+ } else {
+ $metadata->namespace = "";
+ }
+ return $metadata;
}
/**
diff --git a/lib/Doctrine/ORM/Tools/EntityGenerator.php b/lib/Doctrine/ORM/Tools/EntityGenerator.php
index b32197895..b0c83203d 100644
--- a/lib/Doctrine/ORM/Tools/EntityGenerator.php
+++ b/lib/Doctrine/ORM/Tools/EntityGenerator.php
@@ -1,7 +1,5 @@
diff --git a/lib/Doctrine/ORM/Persisters/BasicEntityPersister.php b/lib/Doctrine/ORM/Persisters/BasicEntityPersister.php
index 6ca95c0da..8e9819109 100644
--- a/lib/Doctrine/ORM/Persisters/BasicEntityPersister.php
+++ b/lib/Doctrine/ORM/Persisters/BasicEntityPersister.php
@@ -251,6 +251,8 @@ class BasicEntityPersister
$sql = "SELECT " . $versionFieldColumnName . " FROM " . $class->getQuotedTableName($this->_platform)
. " WHERE " . implode(' = ? AND ', $identifier) . " = ?";
$value = $this->_conn->fetchColumn($sql, array_values((array)$id));
+
+ $value = Type::getType($class->fieldMappings[$versionField]['type'])->convertToPHPValue($value, $this->_platform);
$this->_class->setFieldValue($entity, $versionField, $value);
}
@@ -273,7 +275,15 @@ class BasicEntityPersister
$updateData = $this->_prepareUpdateData($entity);
$tableName = $this->_class->table['name'];
if (isset($updateData[$tableName]) && $updateData[$tableName]) {
- $this->_updateTable($entity, $tableName, $updateData[$tableName], $this->_class->isVersioned);
+ $this->_updateTable(
+ $entity, $this->_class->getQuotedTableName($this->_platform),
+ $updateData[$tableName], $this->_class->isVersioned
+ );
+
+ if ($this->_class->isVersioned) {
+ $id = $this->_em->getUnitOfWork()->getEntityIdentifier($entity);
+ $this->_assignDefaultVersionValue($this->_class, $entity, $id);
+ }
}
}
@@ -282,11 +292,11 @@ class BasicEntityPersister
* The UPDATE can optionally be versioned, which requires the entity to have a version field.
*
* @param object $entity The entity object being updated.
* @param string $tableName The name of the table to apply the UPDATE on.
* @param array $updateData The map of columns to update (column => value).
* @param boolean $versioned Whether the UPDATE should be versioned.
*/
- protected final function _updateTable($entity, $tableName, array $updateData, $versioned = false)
+ protected final function _updateTable($entity, $quotedTableName, array $updateData, $versioned = false)
{
$set = $params = $types = array();
@@ -317,7 +327,7 @@ class BasicEntityPersister
if ($versioned) {
$versionField = $this->_class->versionField;
- $versionFieldType = $this->_class->getTypeOfField($versionField);
+ $versionFieldType = $this->_class->fieldMappings[$versionField]['type'];
$versionColumn = $this->_class->getQuotedColumnName($versionField, $this->_platform);
if ($versionFieldType == Type::INTEGER) {
$set[] = $versionColumn . ' = ' . $versionColumn . ' + 1';
@@ -329,7 +339,7 @@ class BasicEntityPersister
$types[] = $this->_class->fieldMappings[$versionField]['type'];
}
- $sql = "UPDATE $tableName SET " . implode(', ', $set)
+ $sql = "UPDATE $quotedTableName SET " . implode(', ', $set)
. ' WHERE ' . implode(' = ? AND ', $where) . ' = ?';
$result = $this->_conn->executeUpdate($sql, $params, $types);
@@ -393,17 +403,7 @@ class BasicEntityPersister
$this->deleteJoinTableRecords($identifier);
$id = array_combine($this->_class->getIdentifierColumnNames(), $identifier);
- $this->_conn->delete($this->_class->table['name'], $id);
- }
-
- /**
- * Gets the ClassMetadata instance of the entity class this persister is used for.
- *
- * @return Doctrine\ORM\Mapping\ClassMetadata
- */
- public function getClassMetadata()
- {
- return $this->_class;
+ $this->_conn->delete($this->_class->getQuotedTableName($this->_platform), $id);
}
/**
@@ -660,7 +660,7 @@ class BasicEntityPersister
if ($found = $this->_em->getUnitOfWork()->tryGetById($joinColumnValues, $targetClass->rootEntityName)) {
$this->_class->reflFields[$field]->setValue($entity, $found);
// Complete inverse side, if necessary.
- if ($assoc['inversedBy']) {
+ if ($assoc['inversedBy'] && $assoc['type'] & ClassMetadata::ONE_TO_ONE) {
$inverseAssoc = $targetClass->associationMappings[$assoc['inversedBy']];
$targetClass->reflFields[$inverseAssoc['fieldName']]->setValue($found, $entity);
}
@@ -681,6 +681,9 @@ class BasicEntityPersister
}
} else if ($value instanceof PersistentCollection && $value->isInitialized()) {
$value->setInitialized(false);
+ // no matter if dirty or non-dirty entities are already loaded, smoke them out!
+ // the beauty of it being, they are still in the identity map
+ $value->unwrap()->clear();
$newData[$field] = $value;
}
}
@@ -848,8 +851,8 @@ class BasicEntityPersister
}
return 'SELECT ' . $this->_getSelectColumnListSQL()
- . ' FROM ' . $this->_class->getQuotedTableName($this->_platform) . ' '
- . $this->_getSQLTableAlias($this->_class->name)
+ . $this->_platform->appendLockHint(' FROM ' . $this->_class->getQuotedTableName($this->_platform) . ' '
+ . $this->_getSQLTableAlias($this->_class->name), $lockMode)
. $joinSql
. ($conditionSql ? ' WHERE ' . $conditionSql : '')
. $orderBySql
@@ -1086,7 +1089,7 @@ class BasicEntityPersister
}
$sql = 'SELECT 1 '
- . $this->getLockTablesSql()
+ . $this->_platform->appendLockHint($this->getLockTablesSql(), $lockMode)
. ($conditionSql ? ' WHERE ' . $conditionSql : '') . ' ' . $lockSql;
$params = array_values($criteria);
$this->_conn->executeQuery($sql, $params);
@@ -1128,9 +1131,16 @@ class BasicEntityPersister
}
$conditionSql .= $this->_class->getQuotedColumnName($field, $this->_platform);
} else if (isset($this->_class->associationMappings[$field])) {
- // TODO: Inherited?
// TODO: Composite Keys as Foreign Key PK? That would be super ugly! And should probably be disallowed ;)
- $conditionSql .= $this->_getSQLTableAlias($this->_class->name) . '.';
+ if (!$this->_class->associationMappings[$field]['isOwningSide']) {
+ throw ORMException::invalidFindByInverseAssociation($this->_class->name, $field);
+ }
+
+ if (isset($this->_class->associationMappings[$field]['inherited'])) {
+ $conditionSql .= $this->_getSQLTableAlias($this->_class->associationMappings[$field]['inherited']) . '.';
+ } else {
+ $conditionSql .= $this->_getSQLTableAlias($this->_class->name) . '.';
+ }
$conditionSql .= $this->_class->associationMappings[$field]['joinColumns'][0]['name'];
} else if ($assoc !== null) {
@@ -282,11 +292,11 @@ class BasicEntityPersister
*
* @param object $entity The entity object being updated.
- * @param string $tableName The name of the table to apply the UPDATE on.
+ * @param string $quotedTableName The quoted name of the table to apply the UPDATE on.
* @param array $updateData The map of columns to update (column => value).
* @param boolean $versioned Whether the UPDATE should be versioned.